from django.core.exceptions import ObjectDoesNotExist
from django.db import transaction

from compensation.models import Compensation
from intervention.models import Responsibility, Handler, Intervention
from konova.management.commands.kspMigrater.base_migrater import BaseMigrater


class CompensationMigrater(BaseMigrater):

    def migrate(self):
        cursor = self.db_connection.cursor()
        empty_str = "''"
        cursor.execute(
            'select '
                'om."KENNUNG", '
                'linf."OBJBEZ", '
                'ST_AsEWKT(ST_Multi(ST_CollectionExtract(ST_MakeValid(ST_Transform(geomf.the_geom,4326)), 3))) as geomf, '
                'ST_AsEWKT(ST_Multi(ST_CollectionExtract(ST_MakeValid(ST_Transform(geoml.the_geom,4326)), 2))) as geoml, '
                'ST_AsEWKT(ST_Multi(ST_CollectionExtract(ST_MakeValid(ST_Transform(geomp.the_geom,4326)), 1))) as geomp, '
                'linf."Bemerkung" '
            'from "OBJ_MASTER" om '
                'left join "LINFOS" linf on om."GISPADID"=linf."GISPADID" '
                'left join kom on om."GISPADID"=kom.gispadid '
                'left join geometry_f geomf on om."GISPADID"=geomf.gispadid '
                'left join geometry_l geoml on om."GISPADID"=geoml.gispadid '
                'left join geometry_p geomp on om."GISPADID"=geomp.gispadid '
                'left join "Aufwertung" auf on om."GISPADID"=auf."GISPADID" '
            'where '
                'om."OKL"=7730080 and '
                'om.archiv=false and '
                f'(auf."Infos" is null or auf."Infos"={empty_str}) and '
                'om.nicht_vollstaendig=0;'
        )

        all_koms = cursor.fetchall()
        len_all_koms = len(all_koms)
        num_processed = 0
        print(f"Migrate KOMs to compensations...")
        print(f"--Found {len_all_koms} entries. Process now...")
        unsuccessfull_compensations = {}
        for kom in all_koms:
            if num_processed % 500 == 0:
                print(f"----{num_processed}/{len_all_koms} processed")

            with transaction.atomic():
                kom_identifier = kom[0]
                kom_title = kom[1]
                kom_comment = kom[5]
                compensation = Compensation.objects.get_or_create(
                    identifier=kom_identifier
                )[0]

                compensation.title = kom_title
                compensation.comment = kom_comment
                compensation = self._migrate_geometry(compensation, kom)
                compensation = self._migrate_responsibility(compensation, kom)
                try:
                    compensation = self._migrate_interventions_reference(compensation, kom)
                    compensation.save()
                except ObjectDoesNotExist:
                    unsuccessfull_compensations[kom_identifier] = "EIV does not exist"
                    print(f"{kom_identifier} does not have any intervention. ({len(unsuccessfull_compensations)} similar problems)")
                num_processed += 1
        print(f"The following KOMs could not be migrated: {unsuccessfull_compensations}")
        cursor.close()

    def _migrate_interventions_reference(self, compensation, kom):
        kom_identifier = f"'{kom[0]}'"
        tmp_cursor = self.db_connection.cursor()
        tmp_cursor.execute(
            'select '
                'ref."REFERENZ" '
            'from "OBJ_MASTER" om '
            'left join "LINFOS" linf on om."GISPADID"=linf."GISPADID" '
            'left join "REFERENZ" ref on om."GISPADID"=ref."GISPADID" '
            'where '
                'om."OKL"=7730080 and '
                f'om."KENNUNG"={kom_identifier};'
        )
        eivs = tmp_cursor.fetchall()
        len_eivs = len(eivs)
        if len_eivs != 1:
            raise AssertionError(f"{kom_identifier} has {len_eivs} EIVs!")
        eiv = eivs[0]
        try:
            intervention = Intervention.objects.get(
                identifier=eiv[0]
            )
        except ObjectDoesNotExist:
            raise ObjectDoesNotExist(f"{kom_identifier} could not find migrated {eiv}")
        compensation.intervention = intervention
        tmp_cursor.close()
        return compensation

    def _migrate_responsibility(self, compensation, kom):
        compensation.responsible = compensation.responsible or Responsibility.objects.create()
        return compensation